import React from 'react'
import moment from 'moment'
import { Paginator } from './paginator'
import { SymbolBackward, SymbolForward } from './svgs'

function normalize(value, defaultValue) {
  if (value === null || value === undefined || value === false) {
    return defaultValue
  }
  return value
}

function truncate(input_, length_, killwords, end) {
  let input = normalize(input_, '')
  const length = length_ || 255

  if (input.length <= length) {
    return input
  }

  if (killwords) {
    input = input.substring(0, length)
  } else {
    let idx = input.lastIndexOf(' ', length)
    if (idx === -1) {
      idx = length
    }

    input = input.substring(0, idx)
  }

  input += end !== undefined && end !== null ? end : '...'
  return input
}

function trim(str) {
  return str.replace(/^\s*|\s*$/g, '')
}

function striptags(input_, preserveLinebreaks) {
  const input = normalize(input_, '')
  const tags = /<\/?([a-z][a-z0-9]*)\b[^>]*>|<!--[\s\S]*?-->/gi
  const trimmedInput = trim(input.replace(tags, ''))
  let res = ''
  if (preserveLinebreaks) {
    res = trimmedInput
      .replace(/^ +| +$/gm, '') // remove leading and trailing spaces
      .replace(/ +/g, ' ') // squash adjacent spaces
      .replace(/(\r\n)/g, '\n') // normalize linebreaks (CRLF -> LF)
      .replace(/\n\n\n+/g, '\n\n') // squash abnormal adjacent linebreaks
  } else {
    res = trimmedInput.replace(/\s+/gi, ' ')
  }
  return res
}

function dateFilter(date, format) {
  let result
  const errs = []
  const args = []
  let obj
  Array.prototype.push.apply(args, arguments)
  try {
    obj = moment.utc(date)
  } catch (err) {
    errs.push(err)
  }
  if (obj) {
    try {
      if (obj[format] && typeof obj[format] === 'function') {
        result = obj[format].apply(obj, args.slice(2))
      } else {
        result = obj.format(format)
      }
    } catch (err) {
      errs.push(err)
    }
  }

  if (errs.length) {
    return errs.join('\n')
  }
  return result
}

function relURL(filename, dir_) {
  let dir = dir_
  if (filename.substr(0, 1) === '/' || filename.match('://')) return filename
  if (dir.substr(0, 1) !== '/') dir = `/${dir}`
  if (filename.match(/\/$/)) return dir + filename
  if (dir.match(/\/$/)) return dir + filename
  return `${dir}/${filename}`
}

export function List({ subtags, tag, content, metadata, file, config }) {
  return (
    <div id="content" class="container">
      <div class="section">
        <div class="row">
          <div class="col">
            <a class="breadcrumb">{tag.text}</a>
            {subtags &&
              subtags.map(subtag => (
                <a
                  href={`${config.baseurl}/${subtag.url}`}
                  class="chip"
                  key={subtag.url}
                >
                  {subtag.text}
                </a>
              ))}
          </div>
        </div>
        <div class="row">
          {content.map(page => (
            <div class="col s12 m6 l4" key={page.file}>
              <div class="block article">
                <a href={`${config.baseurl}/${page.file}`}>
                  <div
                    style={{
                      height: 120,
                      backgroundImage: page.metadata.image
                        ? `url('${config.baseurl}${relURL(
                            page.metadata.image,
                            page.file,
                          )}')`
                        : undefined,
                    }}
                    className="leadimage light-blue darken-2"
                  ></div>
                </a>
                <div class="head-title">
                  <a href={`${config.baseurl}/${page.file}`}>
                    <h5 class="center">{page.metadata.title}</h5>
                  </a>
                </div>
                <div style={{ padding: '0px 10px' }} className="head-perex">
                  {page.metadata.perex
                    ? page.metadata.perex
                    : page.metadata.perex_e
                    ? truncate(page.metadata.perex_e)
                    : page.content
                    ? truncate(striptags(page.content))
                    : null}
                </div>
                <div class="clear"></div>
                <div style={{ padding: 10 }} className="head-info">
                  <span
                    style={{
                      color: '#a2a2a2',
                      textAlign: 'left',
                      display: 'inline-block',
                      width: '75%',
                    }}
                  >
                    {dateFilter(page.metadata.published, 'D. M. YYYY')}} napsal
                    {page.metadata.author.gender === 'F' ? 'a' : ''}{' '}
                    {page.metadata.author.name}
                  </span>
                  <a
                    style={{
                      textAlign: 'right',
                      display: 'inline-block',
                      width: '25%',
                    }}
                    href={`${config.baseurl}/${page.file}`}
                  >
                    Číst&nbsp;dál
                  </a>
                </div>
              </div>
            </div>
          ))}
        </div>
        <Paginator metadata={metadata} file={file} config={config} />
      </div>

      <div class="hide-on-small-only">
        {metadata.prevurl && (
          <div style={{ left: 0 }} class="sidelink sidelink-l">
            <a href={`${config.baseurl}/${metadata.prevurl}`}>
              <div>
                <SymbolBackward />
              </div>
            </a>
          </div>
        )}
        {metadata.nexturl && (
          <div style={{ right: 0 }} class="sidelink sidelink-r">
            <a href={`${config.baseurl}/${metadata.nexturl}`}>
              <div>
                <SymbolForward />
              </div>
            </a>
          </div>
        )}
      </div>

      <script
        dangerouslySetInnerHTML={{
          __html: `
    (function($) {
      var onresize = function(){
        // equalize article sizes
        $(".head-perex").height("auto");
        var maxHeight = 0;
        $(".head-perex").each(function(){
          if ($(this).height() > maxHeight) { maxHeight = $(this).height(); }
        });
        $(".head-perex").height(maxHeight);

        $(".head-title").height("auto");
        maxHeight = 0;
        $(".head-title").each(function(){
          if ($(this).height() > maxHeight) { maxHeight = $(this).height(); }
        });
        $(".head-title").height(maxHeight);

        $(".head-info").height("auto");
        maxHeight = 0;
        $(".head-info").each(function(){
          if ($(this).height() > maxHeight) { maxHeight = $(this).height(); }
        });
        $(".head-info").height(maxHeight);

        // Sidelink icons of same size
        $(".sidelink-icon").css("top",$(window).height()/2);
      };
      $(window).resize(onresize);
      $(document).load(onresize);
      $(document).ready(onresize);
      $(document).keyup(function(e) {
        if($("#search-overlay").hasClass("shown")) return;
        ${
          metadata.nexturl
            ? `if(e.key == "ArrowRight") window.location = "${config.baseurl}/${metadata.nexturl}";`
            : ''
        }
        ${
          metadata.prevurl
            ? `if(e.key == "ArrowLeft") window.location = "${config.baseurl}/${metadata.prevurl}"`
            : ''
        }
      });
    })(jQuery);
    `,
        }}
      />
    </div>
  )
}
